'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      ClassDeclaration: function ClassDeclaration(path) {
        var node = path.node;
        var classRef = node.id;
        var classBody = node.body.body;

        // Create additional statements for types annotations.
        var types = [];
        classBody.forEach(function (bodyNode) {
          if (bodyNode.type === 'ClassMethod' && bodyNode.kind === 'constructor') {
            types = parameterTypes(bodyNode.params, classRef);
          } else if (bodyNode.type === 'ClassProperty' && bodyNode.value === null && !bodyNode.static) {
            // Handle class property without initializer.
            // https://github.com/jeffmo/es-class-fields-and-static-properties
            bodyNode.value = t.memberExpression(t.thisExpression(), bodyNode.key);
          }
        });
        var additionalStatements = types.filter(Boolean);

        // If not found, do nothing.
        if (additionalStatements.length === 0) {
          return undefined;
        }

        // Append additional statements to program.
        var parent = path.parent;
        if (parent.type === 'ExportNamedDeclaration' || parent.type === 'ExportDefaultDeclaration') {
          // The class declaration is wrapped by an export declaration.
          path.parentPath.insertAfter(additionalStatements);
        } else {
          path.insertAfter(additionalStatements);
        }
      }
    }
  };

  // Returns an array of define 'parameters' metadata statements for a class.
  // The array may contain zero or one statements.
  function parameterTypes(params, classRef) {
    var types = params.map(function (param) {
      var annotation = param.typeAnnotation && param.typeAnnotation.typeAnnotation;
      return typeForAnnotation(annotation);
    });
    if (!types.some(Boolean)) {
      return [];
    }
    return [defineMetadata('design:paramtypes', t.arrayExpression(types), classRef)];
  }

  // Returns an AST for define metadata statement.
  function defineMetadata(key, value, target) {
    return t.expressionStatement(t.callExpression(t.memberExpression(t.identifier('Reflect'), t.identifier('defineMetadata')), [t.stringLiteral(key), value, target]));
  }

  function typeForAnnotation(annotation) {
    if (!annotation) {
      return null;
    }
    switch (annotation.type) {
      case 'StringTypeAnnotation':
        return t.identifier('String');
      case 'NumberTypeAnnotation':
        return t.identifier('Number');
      case 'BooleanTypeAnnotation':
        return t.identifier('Boolean');
      case 'VoidTypeAnnotation':
        return t.unaryExpression('void', t.numericLiteral(0));
      case 'GenericTypeAnnotation':
        if (annotation.typeParameters) {
          var genericCode = (0, _babelGenerator2.default)(annotation).code;
          var plainCode = (0, _babelGenerator2.default)(annotation.id).code;
          console.warn(TAG + ' Generic type is not supported: ' + genericCode + ' Just use: ' + plainCode);
        }
        return annotation.id;
      case 'ObjectTypeAnnotation':
        return t.identifier('Object');
      case 'FunctionTypeAnnotation':
        return t.identifier('Function');
      default:
        return null;
    }
  }
};

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TAG = '[babel-plugin-angular2-annotations]';